----------------------------------------------------------------------------------
-- Company:             www.kampis-elektroecke.de
-- Engineer:            Daniel Kampert
-- 
-- Create Date:         02.08.2014 13:17:51
-- Design Name: 
-- Module Name:         VGA - VGA_Arch
-- Project Name: 
-- Target Devices:      XC7Z010CLG400-1
-- Tool Versions:       Vivado 2014.4
-- Description:         VGA Interface for a 640 x 480 Pixel Screen
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity VGA is
    Generic (
            Color_Width         : integer := 16;
            ROM_Address_Width   : integer := 11;
            ROM_Data_Width      : integer := 8;
            Char_Size           : integer := 8
            );
    Port (  Clock_VGA   : in STD_LOGIC;                                         -- Taktsignal fr VGA-Logik
            Reset       : in STD_LOGIC;                                         -- Reset fr VGA-Logik
            Mode        : in STD_LOGIC;                                         -- Auswahl Betriebsmodus, 0 -> Grafik, 1 -> Text
            Color       : in STD_LOGIC_VECTOR(15 downto 0);
            HSync       : out STD_LOGIC;                                        -- Ausgang fr H-Sync Signal
            VSync       : out STD_LOGIC;                                        -- Ausgang fr V-Sync Signal
            RGB         : out STD_LOGIC_VECTOR((Color_Width - 1) downto 0)      -- RGB Ausgang fr das VGA-Signal
            );
end VGA;

architecture VGA_Arch of VGA is

    -- Koordinaten des Zeigers auf dem Bildschirm
    signal Pixel_x      : unsigned(9 downto 0);               
    signal Pixel_y      : unsigned(9 downto 0); 
    signal Pixel_x_delay      : unsigned(9 downto 0);               
    signal Pixel_y_delay      : unsigned(9 downto 0); 
        
    -- Font ROM
    signal ROM_Data     : std_logic_vector(0 to (ROM_Data_Width - 1));
    signal ROM_Address  : std_logic_vector(0 to (ROM_Address_Width - 1));
    signal ROM_Bit      : std_logic;
    signal Row_Addr     : std_logic_vector(2 downto 0);
    signal Col_Addr     : std_logic_vector(2 downto 0);
       
    -- Display RAM
    signal Display_Address  : unsigned(12 downto 0);
    signal Zeichen_Address  : unsigned(6 downto 0);
    signal Offset           : unsigned(12 downto 0);
    signal Character        : std_logic_vector(7 downto 0); 

    -- Einbinden des VGA-Controllers
    component VGA_Timing is
        Port (  HSync : out STD_LOGIC;
                VSync : out STD_LOGIC;
                Clock_VGA : in STD_LOGIC;
                Reset : in STD_LOGIC;
                x_out : out UNSIGNED(9 downto 0);                
                y_out : out UNSIGNED(9 downto 0) 
                );
    end component VGA_Timing;  
    
    -- Font-ROM fr VGA-Controller
    component Font_ROM is
        port (  ROM_Address : in STD_LOGIC_VECTOR (10 downto 0);
                ROM_Clock : in STD_LOGIC;
                ROM_DataOut : out STD_LOGIC_VECTOR (7 downto 0)
                );
    end component Font_ROM;
    
    -- Display RAM fr das Bild    
    component Display_RAM
        Generic (
                Addr_Width : integer := 13;
                Data_Width : integer := 8
                );
        Port (  Clock : in STD_LOGIC;
                Read_Write : in STD_LOGIC;
                Address : in STD_LOGIC_VECTOR((Addr_Width - 1) downto 0);
                Data_Out : out STD_LOGIC_VECTOR ((Data_Width - 1) downto 0);
                Data_In : in STD_LOGIC_VECTOR ((Data_Width - 1) downto 0)
                );
    end component;

begin

    VGA_Ctrl        : VGA_Timing port map (HSync, VSync, Clock_VGA, Reset, Pixel_x, Pixel_y);   
    Char_Font       : Font_ROM port map (ROM_Address, Clock_VGA, ROM_Data);
    Display         : Display_RAM port map (Clock_VGA, '1', std_logic_vector(Display_Address), Character, "00000000"); 

    -- Farbausgabe
    RGB <= Color when (ROM_Bit = '1') else x"0000";
              
    -- ROM auslesen
    Row_Addr    <= std_logic_vector(Pixel_y(2 downto 0));
    ROM_Address <= Character & Row_Addr;
    Col_Addr    <= std_logic_vector(Pixel_x(2 downto 0));
    ROM_Bit     <= ROM_Data(to_integer(unsigned(Col_Addr)));     
        
    -- Koordinaten um einen Takt verzgern
    process(Clock_VGA)
    begin
        if(rising_edge(Clock_VGA)) then
            Pixel_x_delay <= Pixel_x;
            Pixel_y_delay <= Pixel_y;
        end if;
    end process;

    process(Clock_VGA)
    begin
        if(rising_edge(Clock_VGA)) then
            if(Pixel_x_delay(2 downto 0) = 0) then
                if((Pixel_x_delay = 0) and  (Pixel_y_delay = 0)) then
                    Offset <= to_unsigned(0, Offset'length);
                elsif((Pixel_x_delay = 0) and (Pixel_y_delay(2 downto 0) = 0)) then 
                    Offset <= Offset + 8; 
                end if;        
            end if;    
            
            Zeichen_Address <= Pixel_x_delay(9 downto 3);
            Display_Address <= Zeichen_Address + Offset;
                        
        end if;
    end process;
end VGA_Arch;